import type {
    FoundationListDataProviderFetchMode,
    FoundationListDataProviderResult,
    FoundationListDataProviderResultInitialized,
} from './types';

import listDataFetched from './listDataFetched';


interface TestData {
    text: string;
}

test.each<FoundationListDataProviderFetchMode>([
    'fetching-initial',
    'fetching-sort',
])('should reset after fetching `%s`', (fetchMode) => {
    const list: FoundationListDataProviderResult<TestData> = {
        data: {
            items: [
                {
                    data: {
                        text: 'current',
                    },
                    id: '1',
                },
            ],
            totalCount: 2,
        },
        meta: {
            cursor: null,
            sortBy: 'text',
            sortDirection: 'desc',
        },
        state: fetchMode,
    };

    const newItems = [
        {
            data: {
                text: 'next',
            },
            id: '2',
        },
    ];

    const callback = listDataFetched<TestData>({
        data: {
            items: newItems,
            totalCount: 2,
        },
        meta: {
            cursor: null,
            orderBy: 'text',
            orderDirection: 'desc',
        },
    });

    const result = callback(list);

    expect(result.data.items).toBe(newItems);
});

test('should append `more` items', () => {
    const fetchMode = 'fetching-more';

    const list: FoundationListDataProviderResultInitialized<TestData> = {
        data: {
            items: [
                {
                    data: {
                        text: 'current',
                    },
                    id: '1',
                },
            ],
            totalCount: 2,
        },
        meta: {
            cursor: null,
            sortBy: 'text',
            sortDirection: 'desc',
        },
        state: fetchMode,
    };

    const callback = listDataFetched<TestData>({
        data: {
            items: [{
                data: {
                    text: 'next',
                },
                id: '2',
            }],
            totalCount: 2,
        },
        meta: {
            cursor: null,
            orderBy: 'text',
            orderDirection: 'desc',
        },
    });

    const result = callback(list);

    expect(result).toEqual({
        data: {
            items: [
                {
                    data: {
                        text: 'current',
                    },
                    id: '1',
                },
                {
                    data: {
                        text: 'next',
                    },
                    id: '2',
                },
            ],
            totalCount: 2,
        },
        meta: {
            cursor: null,
            sortBy: 'text',
            sortDirection: 'desc',
        },
        state: 'success',
    });
});

describe.each<FoundationListDataProviderFetchMode>([
    'fetching-initial',
    'fetching-more',
    'fetching-sort',
])('when fetched `%s`', (fetchMode) => {
    const list: FoundationListDataProviderResult<TestData> = {
        data: {
            items: [
                {
                    data: {
                        text: 'current',
                    },
                    id: '1',
                },
            ],
            totalCount: 1,
        },
        meta: {
            cursor: null,
            sortBy: 'unknown',
            sortDirection: 'asc',
        },
        state: fetchMode,
    };

    const newCursor = 'sqYyXsDAwMD24un/klbzSt67/ca+s5OuDkeK95JA+I1mjVPjfr6IBd+KeFtktWNKUAYvfNJTZdYOSKvcWdjN';
    const newOrderBy = 'text';
    const newOrderDirection = 'asc';
    const newTotalCount = 2;

    const callback = listDataFetched<TestData>({
        data: {
            items: [{
                data: {
                    text: 'next',
                },
                id: '2',
            }],
            totalCount: newTotalCount,
        },
        meta: {
            cursor: newCursor,
            orderBy: newOrderBy,
            orderDirection: newOrderDirection,
        },
    });

    const result = callback(list);

    it('should not be fetching', () => {
        expect(result.state).toBe('success');
    });

    it('should have updated `cursor`', () => {
        expect(result.meta.cursor).toBe(newCursor);
    });

    it('should have updated `sortBy`', () => {
        expect(result.meta.sortBy).toBe(newOrderBy);
    });

    it('should have updated `sortDirection`', () => {
        expect(result.meta.sortDirection).toBe(newOrderDirection);
    });

    it('should have updated `totalCount`', () => {
        expect(result.data.totalCount).toBe(newTotalCount);
    });
});
