import listDataUpdateItem from './listDataUpdateItem';

import type {
    FoundationListDataProviderFetchState,
    FoundationListDataProviderResult,
} from './types';


interface TestData {
    text: string;
}

test.each<FoundationListDataProviderFetchState>([
    'error-initial',
    'error-more',
    'error-sort',
    'fetching-initial',
])('should not update item when in %s state', (fetchMode) => {
    const list: FoundationListDataProviderResult<TestData> = {
        data: {
            items: [
                {
                    data: {
                        text: 'current #1',
                    },
                    id: '1',
                },
                {
                    data: {
                        text: 'current #2',
                    },
                    id: '2',
                },
                {
                    data: {
                        text: 'current #3',
                    },
                    id: '3',
                },
            ],
            totalCount: 3,
        },
        meta: {
            cursor: null,
            sortBy: 'text',
            sortDirection: 'desc',
        },
        state: fetchMode,
    };

    const callback = listDataUpdateItem({
        data: {
            text: 'updated #2',
        },
        id: '2',
    });

    const result = callback(list);

    expect(result).toBe(list);
});

test.each<FoundationListDataProviderFetchState>([
    'fetching-more',
    'fetching-sort',
    'success',
])('should ignore update if id mismatch', (fetchMode) => {
    const list: FoundationListDataProviderResult<TestData> = {
        data: {
            items: [
                {
                    data: {
                        text: 'current #1',
                    },
                    id: '1',
                },
                {
                    data: {
                        text: 'current #2',
                    },
                    id: '2',
                },
                {
                    data: {
                        text: 'current #3',
                    },
                    id: '3',
                },
            ],
            totalCount: 3,
        },
        meta: {
            cursor: null,
            sortBy: 'text',
            sortDirection: 'desc',
        },
        state: fetchMode,
    };

    const callback = listDataUpdateItem({
        data: {
            text: 'updated',
        },
        id: '4',
    });

    const result = callback(list);

    expect(result).toBe(list);
});

test.each<FoundationListDataProviderFetchState>([
    'fetching-more',
    'fetching-sort',
    'success',
])('should be able to update first item when in %s state', (fetchMode) => {
    const list: FoundationListDataProviderResult<TestData> = {
        data: {
            items: [
                {
                    data: {
                        text: 'current #1',
                    },
                    id: '1',
                },
                {
                    data: {
                        text: 'current #2',
                    },
                    id: '2',
                },
                {
                    data: {
                        text: 'current #3',
                    },
                    id: '3',
                },
            ],
            totalCount: 3,
        },
        meta: {
            cursor: null,
            sortBy: 'text',
            sortDirection: 'desc',
        },
        state: fetchMode,
    };

    const callback = listDataUpdateItem({
        data: {
            text: 'updated #1',
        },
        id: '1',
    });

    const result = callback(list);

    expect(result).toEqual({
        data: {
            items: [
                {
                    data: {
                        text: 'updated #1',
                    },
                    id: '1',
                },
                {
                    data: {
                        text: 'current #2',
                    },
                    id: '2',
                },
                {
                    data: {
                        text: 'current #3',
                    },
                    id: '3',
                },
            ],
            totalCount: 3,
        },
        meta: {
            cursor: null,
            sortBy: 'text',
            sortDirection: 'desc',
        },
        state: fetchMode,
    });
});

test.each<FoundationListDataProviderFetchState>([
    'fetching-more',
    'fetching-sort',
    'success',
])('should be able to update in-between item when in %s state', (fetchMode) => {
    const list: FoundationListDataProviderResult<TestData> = {
        data: {
            items: [
                {
                    data: {
                        text: 'current #1',
                    },
                    id: '1',
                },
                {
                    data: {
                        text: 'current #2',
                    },
                    id: '2',
                },
                {
                    data: {
                        text: 'current #3',
                    },
                    id: '3',
                },
            ],
            totalCount: 3,
        },
        meta: {
            cursor: null,
            sortBy: 'text',
            sortDirection: 'desc',
        },
        state: fetchMode,
    };

    const callback = listDataUpdateItem({
        data: {
            text: 'updated #2',
        },
        id: '2',
    });

    const result = callback(list);

    expect(result).toEqual({
        data: {
            items: [
                {
                    data: {
                        text: 'current #1',
                    },
                    id: '1',
                },
                {
                    data: {
                        text: 'updated #2',
                    },
                    id: '2',
                },
                {
                    data: {
                        text: 'current #3',
                    },
                    id: '3',
                },
            ],
            totalCount: 3,
        },
        meta: {
            cursor: null,
            sortBy: 'text',
            sortDirection: 'desc',
        },
        state: fetchMode,
    });
});

test.each<FoundationListDataProviderFetchState>([
    'fetching-more',
    'fetching-sort',
    'success',
])('should be able to update last item when in %s state', (fetchMode) => {
    const list: FoundationListDataProviderResult<TestData> = {
        data: {
            items: [
                {
                    data: {
                        text: 'current #1',
                    },
                    id: '1',
                },
                {
                    data: {
                        text: 'current #2',
                    },
                    id: '2',
                },
                {
                    data: {
                        text: 'current #3',
                    },
                    id: '3',
                },
            ],
            totalCount: 3,
        },
        meta: {
            cursor: null,
            sortBy: 'text',
            sortDirection: 'desc',
        },
        state: fetchMode,
    };

    const callback = listDataUpdateItem({
        data: {
            text: 'updated #3',
        },
        id: '3',
    });

    const result = callback(list);

    expect(result).toEqual({
        data: {
            items: [
                {
                    data: {
                        text: 'current #1',
                    },
                    id: '1',
                },
                {
                    data: {
                        text: 'current #2',
                    },
                    id: '2',
                },
                {
                    data: {
                        text: 'updated #3',
                    },
                    id: '3',
                },
            ],
            totalCount: 3,
        },
        meta: {
            cursor: null,
            sortBy: 'text',
            sortDirection: 'desc',
        },
        state: fetchMode,
    });
});
