/// <reference types="react" />
export declare type FoundationListDataProviderCursor = string | null;
export declare type FoundationListDataProviderSortDirection = 'asc' | 'desc';
export interface FoundationListDataProviderRowType<TRowData> {
    data: TRowData;
    id: number | string;
}
export interface FoundationListDataProviderData<TRowData> {
    items: Array<FoundationListDataProviderRowType<TRowData>>;
    totalCount: number;
}
export interface FoundationListDataProviderResultMeta {
    cursor: FoundationListDataProviderCursor;
    sortBy: string;
    sortDirection: FoundationListDataProviderSortDirection;
}
export interface FoundationListDataProviderResultInitializing {
    state: 'error-initial' | 'fetching-initial';
}
export interface FoundationListDataProviderResultInitialized<TRowData, TData extends FoundationListDataProviderData<TRowData> = FoundationListDataProviderData<TRowData>> {
    data: TData;
    meta: FoundationListDataProviderResultMeta;
    state: 'error-more' | 'error-sort' | 'fetching-more' | 'fetching-sort' | 'success';
}
export declare type FoundationListDataProviderResult<TRowData, TData extends FoundationListDataProviderData<TRowData> = FoundationListDataProviderData<TRowData>> = FoundationListDataProviderResultInitializing | FoundationListDataProviderResultInitialized<TRowData, TData>;
export declare type FoundationListDataProviderFetchMode = 'fetching-initial' | 'fetching-more' | 'fetching-sort';
export declare type FoundationListDataProviderFetchState = FoundationListDataProviderFetchMode | 'error-initial' | 'error-more' | 'error-sort' | 'success';
export interface FoundationListDataProviderFetchInitialHandler<TOptions = never> {
    (options?: TOptions): void;
}
export interface FoundationListDataProviderFetchMoreHandler<TOptions = never> {
    (limit: number, options?: TOptions): void;
}
export interface FoundationListDataProviderFetchSortHandler<TOptions = never> {
    (sortBy: string, options?: TOptions): void;
}
export interface FoundationListDataProviderFetchParams {
    cursor: FoundationListDataProviderCursor;
    offset: number;
    limit: number;
    sortBy: string;
    sortDirection: FoundationListDataProviderSortDirection;
}
export interface FoundationListDataProviderFetch<TRowData, TData extends FoundationListDataProviderData<TRowData> = FoundationListDataProviderData<TRowData>, TOptions = never> {
    (params: FoundationListDataProviderFetchParams, options?: TOptions): Promise<FoundationListDataProviderFetchResult<TRowData, TData>>;
}
export interface FoundationListDataProviderFetchResult<TRowData, TData extends FoundationListDataProviderData<TRowData> = FoundationListDataProviderData<TRowData>> {
    data: TData;
    meta: {
        cursor: FoundationListDataProviderCursor;
        orderBy: string;
        orderDirection: FoundationListDataProviderSortDirection;
    };
}
export interface FoundationListDataProviderUpdateItemHandler<TRowData> {
    (item: FoundationListDataProviderRowType<TRowData>): void;
}
export interface FoundationListDataProviderRemoveItemHandler<TRowData> {
    (item: FoundationListDataProviderRowType<TRowData>): void;
}
export interface FoundationListDataProviderContextType<TRowData, TData extends FoundationListDataProviderData<TRowData> = FoundationListDataProviderData<TRowData>, TOptions = never> {
    fetchInitial: FoundationListDataProviderFetchInitialHandler<TOptions>;
    fetchMore: FoundationListDataProviderFetchMoreHandler<TOptions>;
    fetchSort: FoundationListDataProviderFetchSortHandler<TOptions>;
    removeItem: FoundationListDataProviderRemoveItemHandler<TRowData>;
    result: FoundationListDataProviderResult<TRowData, TData>;
    updateItem: FoundationListDataProviderUpdateItemHandler<TRowData>;
}
export interface FoundationListDataProviderProps<TRowData, TData extends FoundationListDataProviderData<TRowData> = FoundationListDataProviderData<TRowData>, TOptions = never> {
    children: React.ReactNode;
    fetchData: FoundationListDataProviderFetch<TRowData, TData, TOptions>;
    initialFetchDataOptions?: TOptions;
    initialLimit?: number;
    initialSortBy?: string;
    initialSortDirection?: FoundationListDataProviderSortDirection;
}
export interface FoundationListDataConsumerProps<TRowData, TData extends FoundationListDataProviderData<TRowData> = FoundationListDataProviderData<TRowData>, TOptions = never> {
    children: (context: FoundationListDataProviderContextType<TRowData, TData, TOptions>) => React.ReactElement | null;
}
export interface FoundationListDataProviderOptions {
    cursor: FoundationListDataProviderCursor;
    limit?: number;
    mode: FoundationListDataProviderFetchMode;
    offset: number;
    sortBy: string;
    sortDirection: FoundationListDataProviderSortDirection;
}
export declare type FoundationListDataProsumerProps<TRowData, TData extends FoundationListDataProviderData<TRowData> = FoundationListDataProviderData<TRowData>, TOptions = never> = Omit<FoundationListDataProviderProps<TRowData, TData, TOptions>, 'children'> & FoundationListDataConsumerProps<TRowData, TData, TOptions>;
export interface FoundationDataProviderCreateResult<TRowData, TData extends FoundationListDataProviderData<TRowData> = FoundationListDataProviderData<TRowData>, TOptions = never> {
    FoundationListDataConsumer: React.ComponentType<FoundationListDataConsumerProps<TRowData, TData, TOptions>>;
    FoundationListDataProvider: React.ComponentType<FoundationListDataProviderProps<TRowData, TData, TOptions>>;
    FoundationListDataProsumer: React.ComponentType<FoundationListDataProsumerProps<TRowData, TData, TOptions>>;
    useListDataProvider: () => FoundationListDataProviderContextType<TRowData, TData, TOptions>;
}
